@@ -26,7 +26,7 @@ from oslo_utils import excutils
 from oslo_utils import versionutils as vutils
 import six
 
-from oslo_versionedobjects._i18n import _, _LE
+from oslo_versionedobjects._i18n import _
 from oslo_versionedobjects import exception
 from oslo_versionedobjects import fields as obj_fields
 
@@ -56,7 +56,7 @@ def _make_class_properties(cls):
         for name, field in supercls.fields.items():
             if name not in cls.fields:
                 cls.fields[name] = field
-    for name, field in six.iteritems(cls.fields):
+    for name, field in cls.fields.items():
         if not isinstance(field, obj_fields.Field):
             raise exception.ObjectFieldInvalid(
                 field=name, objname=cls.obj_name())
@@ -84,7 +84,7 @@ def _make_class_properties(cls):
             except Exception:
                 with excutils.save_and_reraise_exception():
                     attr = "%s.%s" % (self.obj_name(), name)
-                    LOG.exception(_LE('Error setting %(attr)s'),
+                    LOG.exception('Error setting %(attr)s',
                                   {'attr': attr})
 
         def deleter(self, name=name):
@@ -208,7 +208,7 @@ def remotable(fn):
         if self.indirection_api:
             updates, result = self.indirection_api.object_action(
                 ctxt, self, fn.__name__, args, kwargs)
-            for key, value in six.iteritems(updates):
+            for key, value in updates.items():
                 if key in self.fields:
                     field = self.fields[key]
                     # NOTE(ndipanov): Since VersionedObjectSerializer will have
@@ -314,7 +314,7 @@ class VersionedObject(object):
                                   field.stringify(getattr(self, name)) or
                                   '<?>'))
                       for name, field in sorted(self.fields.items())]))
-        if not six.PY3:
+        if six.PY2:
             repr_str = encodeutils.safe_encode(repr_str, incoming='utf-8')
         return repr_str
 
@@ -327,45 +327,11 @@ class VersionedObject(object):
     @classmethod
     def to_json_schema(cls):
         obj_name = cls.obj_name()
-        field_schemas = {key: field.get_schema()
-                         for key, field in cls.fields.items()}
-        required_fields = [name for name, schema in
-                           sorted(field_schemas.items())]
-        namespace_key = cls._obj_primitive_key('namespace')
-        name_key = cls._obj_primitive_key('name')
-        version_key = cls._obj_primitive_key('version')
-        data_key = cls._obj_primitive_key('data')
-        changes_key = cls._obj_primitive_key('changes')
-
         schema = {
             '$schema': 'http://json-schema.org/draft-04/schema#',
             'title': obj_name,
-            'type': 'object',
-            'properties': {
-                namespace_key: {
-                    'type': 'string'
-                },
-                name_key: {
-                    'type': 'string'
-                },
-                version_key: {
-                    'type': 'string'
-                },
-                changes_key: {
-                    'type': 'array',
-                    'items': {
-                        'type': 'string'
-                    }
-                },
-                data_key: {
-                    'type': 'object',
-                    'description': 'fields of %s' % (obj_name),
-                    'properties': field_schemas,
-                    'required': required_fields
-                },
-            },
-            'required': [namespace_key, name_key, version_key, data_key]
         }
+        schema.update(obj_fields.Object(obj_name).get_schema())
         return schema
 
     @classmethod
@@ -394,8 +360,8 @@ class VersionedObject(object):
     def obj_class_from_name(cls, objname, objver):
         """Returns a class from the registry based on a name and version."""
         if objname not in VersionedObjectRegistry.obj_classes():
-            LOG.error(_LE('Unable to instantiate unregistered object type '
-                          '%(objtype)s'), dict(objtype=objname))
+            LOG.error('Unable to instantiate unregistered object type '
+                      '%(objtype)s'), dict(objtype=objname)
             raise exception.UnsupportedObjectError(objtype=objname)
 
         # NOTE(comstud): If there's not an exact match, return the highest
@@ -458,6 +424,11 @@ class VersionedObject(object):
         # of issues by copying only our field data.
 
         nobj = self.__class__()
+
+        # NOTE(sskripnick): we should save newly created object into mem
+        # to let deepcopy know which branches are already created.
+        # See launchpad bug #1602314 for more details
+        memo[id(self)] = nobj
         nobj._context = self._context
         for name in self.fields:
             if self.obj_attr_is_set(name):
@@ -735,12 +706,27 @@ class ComparableVersionedObject(object):
             return self.obj_to_primitive() == obj.obj_to_primitive()
         return NotImplemented
 
+    def __hash__(self):
+        return super(ComparableVersionedObject, self).__hash__()
+
     def __ne__(self, obj):
         if hasattr(obj, 'obj_to_primitive'):
             return self.obj_to_primitive() != obj.obj_to_primitive()
         return NotImplemented
 
 
+class TimestampedObject(object):
+    """Mixin class for db backed objects with timestamp fields.
+
+    Sqlalchemy models that inherit from the oslo_db TimestampMixin will include
+    these fields and the corresponding objects will benefit from this mixin.
+    """
+    fields = {
+        'created_at': obj_fields.DateTimeField(nullable=True),
+        'updated_at': obj_fields.DateTimeField(nullable=True),
+    }
+
+
 class VersionedObjectDictCompat(object):
     """Mix-in to provide dictionary key access compatibility
 
@@ -880,6 +866,28 @@ class ObjectListBase(collections.Sequence):
                 changes.add('objects')
         return changes
 
+    def __add__(self, other):
+        # Handling arbitrary fields may not make sense if those fields are not
+        # all concatenatable. Only concatenate if the base 'objects' field is
+        # the only one and the classes match.
+        if (self.__class__ == other.__class__ and
+                list(self.__class__.fields.keys()) == ['objects']):
+            return self.__class__(objects=self.objects + other.objects)
+        else:
+            raise TypeError("List Objects should be of the same type and only "
+                            "have an 'objects' field")
+
+    def __radd__(self, other):
+        if (self.__class__ == other.__class__ and
+                list(self.__class__.fields.keys()) == ['objects']):
+            # This should never be run in practice. If the above condition is
+            # met then __add__ would have been run.
+            raise NotImplementedError('__radd__ is not implemented for '
+                                      'objects of the same type')
+        else:
+            raise TypeError("List Objects should be of the same type and only "
+                            "have an 'objects' field")
+
 
 class VersionedObjectSerializer(messaging.NoOpSerializer):
     """A VersionedObject-aware Serializer.
@@ -940,7 +948,7 @@ class VersionedObjectSerializer(messaging.NoOpSerializer):
         iterable = values.__class__
         if issubclass(iterable, dict):
             return iterable([(k, action_fn(context, v))
-                             for k, v in six.iteritems(values)])
+                             for k, v in values.items()])
         else:
             # NOTE(danms, gibi) A set can't have an unhashable value inside,
             # such as a dict. Convert the set to list, which is fine, since we
@@ -1136,7 +1144,7 @@ def obj_tree_get_versions(objname, tree=None):
     level object with other objects as fields. It walks the tree recursively
     to determine all the objects (by symbolic name) that could be contained
     within the top-level object, and the maximum versions of each. The result
-    is a dict like:
+    is a dict like::
 
       {'MyObject': '1.23', ... }
 
