@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 #    Copyright 2013 Red Hat, Inc.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -13,6 +14,7 @@
 #    under the License.
 
 import datetime
+import warnings
 
 import iso8601
 import mock
@@ -158,7 +160,7 @@ class TestString(TestField):
         self.coerce_good_values = [
             ('foo', 'foo'), (1, '1'), (1.0, '1.0'), (True, 'True')]
         if six.PY2:
-            self.coerce_good_values += [(long(1), '1')]
+            self.coerce_good_values += [(long(1), '1')]  # noqa
         self.coerce_bad_values = [None]
         self.to_primitive_values = self.coerce_good_values[0:1]
         self.from_primitive_values = self.coerce_good_values[0:1]
@@ -284,6 +286,9 @@ class TestUUID(TestField):
             ('da66a411-af0e-4829-9b67-475017ddz152',
                 'da66a411-af0e-4829-9b67-475017ddz152'),
             ('fake_uuid', 'fake_uuid'),
+            (u'fake_uāid', u'fake_uāid'),
+            (b'fake_u\xe1id'.decode('latin_1'),
+                b'fake_u\xe1id'.decode('latin_1')),
             ('1', '1'),
             (1, '1')
         ]
@@ -296,13 +301,17 @@ class TestUUID(TestField):
         self.test_from_primitive()
         self.test_to_primitive()
 
+    def test_validation_warning_can_be_escalated_to_exception(self):
+        warnings.filterwarnings(action='error')
+        self.assertRaises(FutureWarning, self.field.coerce, 'obj', 'attr',
+                          'not a uuid')
+
     def test_get_schema(self):
         field = fields.UUIDField()
         schema = field.get_schema()
         self.assertEqual(['string'], schema['type'])
         self.assertEqual(False, schema['readonly'])
         pattern = schema['pattern']
-        print(self.coerce_good_values)
         for _, valid_val in self.coerce_good_values[:4]:
             self.assertRegex(valid_val, pattern)
         invalid_vals = [x for x in self.coerce_bad_values if type(x) == 'str']
@@ -360,11 +369,11 @@ class TestEnum(TestField):
         self.assertRaises(ValueError, self.field.stringify, '123')
 
     def test_fieldtype_get_schema(self):
-        self.assertEqual({'enum': ["foo", "bar", 1, True]},
+        self.assertEqual({'type': ['string'], 'enum': ["foo", "bar", 1, True]},
                          self.field._type.get_schema())
 
     def test_get_schema(self):
-        self.assertEqual({'enum': ["foo", "bar", 1, True],
+        self.assertEqual({'type': ['string'], 'enum': ["foo", "bar", 1, True],
                           'readonly': False}, self.field.get_schema())
 
     def test_fingerprint(self):
@@ -487,6 +496,10 @@ class TestNonNegativeInteger(TestField):
         self.to_primitive_values = self.coerce_good_values[0:1]
         self.from_primitive_values = self.coerce_good_values[0:1]
 
+    def test_get_schema(self):
+        self.assertEqual({'type': ['integer'], 'readonly': False,
+                          'minimum': 0}, self.field.get_schema())
+
 
 class TestFloat(TestField):
     def setUp(self):
@@ -514,6 +527,10 @@ class TestNonNegativeFloat(TestField):
         self.to_primitive_values = self.coerce_good_values[0:1]
         self.from_primitive_values = self.coerce_good_values[0:1]
 
+    def test_get_schema(self):
+        self.assertEqual({'type': ['number'], 'readonly': False,
+                          'minimum': 0}, self.field.get_schema())
+
 
 class TestBoolean(TestField):
     def setUp(self):
@@ -553,7 +570,7 @@ class TestFlexibleBoolean(TestField):
 class TestDateTime(TestField):
     def setUp(self):
         super(TestDateTime, self).setUp()
-        self.dt = datetime.datetime(1955, 11, 5, tzinfo=iso8601.iso8601.Utc())
+        self.dt = datetime.datetime(1955, 11, 5, tzinfo=iso8601.iso8601.UTC)
         self.field = fields.DateTimeField()
         self.coerce_good_values = [(self.dt, self.dt),
                                    (_utils.isotime(self.dt), self.dt)]
@@ -566,7 +583,7 @@ class TestDateTime(TestField):
             '1955-11-05T18:00:00Z',
             self.field.stringify(
                 datetime.datetime(1955, 11, 5, 18, 0, 0,
-                                  tzinfo=iso8601.iso8601.Utc())))
+                                  tzinfo=iso8601.iso8601.UTC)))
 
     def test_get_schema(self):
         self.assertEqual({'type': ['string'], 'format': 'date-time',
@@ -611,7 +628,10 @@ class TestDict(TestField):
         self.assertEqual("{key=val}", self.field.stringify({'key': 'val'}))
 
     def test_get_schema(self):
-        self.assertEqual({'type': ['object'], 'readonly': False},
+        self.assertEqual({'type': ['object'],
+                          'additionalProperties': {'readonly': False,
+                                                   'type': ['foo']},
+                          'readonly': False},
                          self.field.get_schema())
 
 
@@ -793,6 +813,15 @@ class TestSetOfIntegers(TestField):
     def test_stringify(self):
         self.assertEqual('set([1,2])', self.field.stringify(set([1, 2])))
 
+    def test_repr(self):
+        self.assertEqual("Set(default=<class 'oslo_versionedobjects.fields."
+                         "UnspecifiedDefault'>,nullable=False)",
+                         repr(self.field))
+        self.assertEqual("Set(default=set([]),nullable=False)",
+                         repr(fields.SetOfIntegersField(default=set())))
+        self.assertEqual("Set(default=set([1,a]),nullable=False)",
+                         repr(fields.SetOfIntegersField(default={1, 'a'})))
+
 
 class TestListOfSetsOfIntegers(TestField):
     def setUp(self):
@@ -901,7 +930,7 @@ class TestObject(TestField):
         class TestFoo(TestFakeObject, obj_base.ComparableVersionedObject):
             fields = {
                 'name': fields.StringField(),
-                'bar': fields.ObjectField('TestBar')
+                'bar': fields.ObjectField('TestBar', nullable=True)
             }
 
         bar = TestBar(name='bar')
@@ -968,6 +997,12 @@ class TestObject(TestField):
                           "TestAnimal", "VersionedObject"],
                          fields.Object._get_all_obj_names(babe))
 
+        # When stringifying we should see the subclass object name
+        # not the base class object name
+        self.assertEqual("TestDog", animals.stringify(wolfy))
+        self.assertEqual("TestCrocodile", animals.stringify(ticktock))
+        self.assertEqual("TestPig", animals.stringify(babe))
+
         # Everything is an animal
         self.assertEqual(wolfy, animals.coerce(None, "animal", wolfy))
         self.assertEqual(ticktock, animals.coerce(None, "animal", ticktock))
@@ -1010,6 +1045,31 @@ class TestObject(TestField):
         self.assertEqual('An object of type TestableObject is required '
                          'in field attr, not a list', six.text_type(ex))
 
+    def test_get_schema(self):
+        self.assertEqual(
+            {
+                'properties': {
+                    'versioned_object.changes':
+                        {'items': {'type': 'string'}, 'type': 'array'},
+                    'versioned_object.data': {
+                        'description': 'fields of TestableObject',
+                        'properties':
+                            {'uuid': {'readonly': False, 'type': ['string']}},
+                        'required': ['uuid'],
+                        'type': 'object'},
+                    'versioned_object.name': {'type': 'string'},
+                    'versioned_object.namespace': {'type': 'string'},
+                    'versioned_object.version': {'type': 'string'}
+                },
+                'readonly': False,
+                'required': ['versioned_object.namespace',
+                             'versioned_object.name',
+                             'versioned_object.version',
+                             'versioned_object.data'],
+                'type': ['object']
+            },
+            self.field.get_schema())
+
 
 class TestIPAddress(TestField):
     def setUp(self):
@@ -1055,8 +1115,7 @@ class TestIPAddressV6(TestField):
                                     netaddr.IPAddress('::1'))]
         self.coerce_bad_values = ['1.2', 'foo', '1.2.3.4']
         self.to_primitive_values = [(netaddr.IPAddress('::1'), '::1')]
-        self.from_primitive_values = [('::1',
-                                       netaddr.IPAddress('::1'))]
+        self.from_primitive_values = [('::1', netaddr.IPAddress('::1'))]
 
     def test_get_schema(self):
         self.assertEqual({'type': ['string'], 'readonly': False,
@@ -1085,6 +1144,11 @@ class TestIPV4AndV6Address(TestField):
                                       ('1.2.3.4',
                                        netaddr.IPAddress('1.2.3.4'))]
 
+    def test_get_schema(self):
+        self.assertEqual({'oneOf': [{'format': 'ipv4', 'type': ['string']},
+                                    {'format': 'ipv6', 'type': ['string']}]},
+                         self.field.get_schema())
+
 
 class TestIPNetwork(TestField):
     def setUp(self):
